import Blockchain from 'neo-blockchain-impl';
import { finalize, neverComplete } from 'neo-blockchain-node-core';
import Node from 'neo-blockchain-node';
import 'rxjs/Observable';
import { rpcServer$ } from 'neo-blockchain-rpc';
import { defer as defer$1 } from 'rxjs/observable/defer';
import { concatMap, distinct, map } from 'rxjs/operators';
import leveldown from 'leveldown';
import levelup from 'levelup';
import levelUpStorage from 'neo-blockchain-levelup';
import vm from 'neo-blockchain-vm';

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve(value);
        } else {
          Promise.resolve(value).then(_next, _throw);
        }
      }

      function _next(value) {
        step("next", value);
      }

      function _throw(err) {
        step("throw", err);
      }

      _next();
    });
  };
}

var fullNode$$1 = (function (_ref) {
  var log = _ref.log,
      createLogForContext = _ref.createLogForContext,
      createProfile = _ref.createProfile,
      settings = _ref.settings,
      environment = _ref.environment,
      options$ = _ref.options$,
      onCreateBlockchain = _ref.onCreateBlockchain;
  return defer$1(_asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee() {
    var storage, blockchain, node;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            storage = levelUpStorage({
              db: levelup(leveldown(environment.dataPath, environment.levelDownOptions)),
              context: {
                messageMagic: settings.messageMagic
              }
            });
            _context.next = 3;
            return Blockchain.create({
              settings: settings,
              storage: storage,
              vm: vm,
              log: log
            });

          case 3:
            blockchain = _context.sent;

            if (!(onCreateBlockchain != null)) {
              _context.next = 7;
              break;
            }

            _context.next = 7;
            return onCreateBlockchain(blockchain);

          case 7:
            node = new Node({
              blockchain: blockchain,
              seeds$: options$.pipe(map(function (options) {
                return options.node.seeds;
              }), distinct())
            });
            node.start();
            return _context.abrupt("return", {
              node: node,
              blockchain: blockchain,
              storage: storage
            });

          case 10:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  }))).pipe(neverComplete(), finalize(function () {
    var _ref3 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee2(result) {
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (!(result != null)) {
                _context2.next = 7;
                break;
              }

              _context2.next = 3;
              return result.node.stop();

            case 3:
              _context2.next = 5;
              return result.blockchain.stop();

            case 5:
              _context2.next = 7;
              return result.storage.close();

            case 7:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function (_x) {
      return _ref3.apply(this, arguments);
    };
  }()), concatMap(function (_ref4) {
    var node = _ref4.node,
        blockchain = _ref4.blockchain;
    return rpcServer$({
      log: log,
      createLogForContext: createLogForContext,
      createProfile: createProfile,
      blockchain: blockchain,
      node: node,
      environment: environment.rpc,
      options$: options$.pipe(map(function (options) {
        return options.rpc;
      }), distinct())
    });
  }));
});

export default fullNode$$1;
//# sourceMappingURL=es.js.map
